@@ -8,9 +8,11 @@ from click import ParamType
 from click.types import StringParamType
 
 from celery import concurrency
-from celery.bin.base import (COMMA_SEPARATED_LIST, LOG_LEVEL,
-                             CeleryDaemonCommand, CeleryOption)
-from celery.platforms import EX_FAILURE, detached, maybe_drop_privileges
+from celery.bin.base import (COMMA_SEPARATED_LIST, LOG_LEVEL, CeleryDaemonCommand, CeleryOption,
+                             handle_preload_options)
+from celery.concurrency.base import BasePool
+from celery.exceptions import SecurityError
+from celery.platforms import EX_FAILURE, EX_OK, detached, maybe_drop_privileges
 from celery.utils.log import get_logger
 from celery.utils.nodenames import default_nodename, host_format, node_format
 
@@ -37,13 +39,28 @@ class WorkersPool(click.Choice):
 
     def __init__(self):
         """Initialize the workers pool option with the relevant choices."""
-        super().__init__(('prefork', 'eventlet', 'gevent', 'solo'))
+        super().__init__(concurrency.get_available_pool_names())
 
     def convert(self, value, param, ctx):
         # Pools like eventlet/gevent needs to patch libs as early
         # as possible.
-        return concurrency.get_implementation(
-            value) or ctx.obj.app.conf.worker_pool
+        if isinstance(value, type) and issubclass(value, BasePool):
+            return value
+
+        value = super().convert(value, param, ctx)
+        worker_pool = ctx.obj.app.conf.worker_pool
+        if value == 'prefork' and worker_pool:
+            # If we got the default pool through the CLI
+            # we need to check if the worker pool was configured.
+            # If the worker pool was configured, we shouldn't use the default.
+            value = concurrency.get_implementation(worker_pool)
+        else:
+            value = concurrency.get_implementation(value)
+
+            if not value:
+                value = concurrency.get_implementation(worker_pool)
+
+        return value
 
 
 class Hostname(StringParamType):
@@ -93,12 +110,18 @@ def detach(path, argv, logfile=None, pidfile=None, uid=None,
            executable=None, hostname=None):
     """Detach program by argv."""
     fake = 1 if C_FAKEFORK else fake
+    # `detached()` will attempt to touch the logfile to confirm that error
+    # messages won't be lost after detaching stdout/err, but this means we need
+    # to pre-format it rather than relying on `setup_logging_subsystem()` like
+    # we can elsewhere.
+    logfile = node_format(logfile, hostname)
     with detached(logfile, pidfile, uid, gid, umask, workdir, fake,
                   after_forkers=False):
         try:
             if executable is not None:
                 path = executable
             os.execv(path, [path] + argv)
+            return EX_OK
         except Exception:  # pylint: disable=broad-except
             if app is None:
                 from celery import current_app
@@ -107,7 +130,7 @@ def detach(path, argv, logfile=None, pidfile=None, uid=None,
                 'ERROR', logfile, hostname=hostname)
             logger.critical("Can't exec %r", ' '.join([path] + argv),
                             exc_info=True)
-        return EX_FAILURE
+            return EX_FAILURE
 
 
 @click.command(cls=CeleryDaemonCommand,
@@ -131,9 +154,10 @@ def detach(path, argv, logfile=None, pidfile=None, uid=None,
               '--statedb',
               cls=CeleryOption,
               type=click.Path(),
-              callback=lambda ctx, _, value: value or ctx.obj.app.conf.worker_state_db,
+              callback=lambda ctx, _,
+              value: value or ctx.obj.app.conf.worker_state_db,
               help_group="Worker Options",
-              help="Path to the state database. The extension '.db' may be"
+              help="Path to the state database. The extension '.db' may be "
                    "appended to the filename.")
 @click.option('-l',
               '--loglevel',
@@ -152,21 +176,23 @@ def detach(path, argv, logfile=None, pidfile=None, uid=None,
 @click.option('--prefetch-multiplier',
               type=int,
               metavar="<prefetch multiplier>",
-              callback=lambda ctx, _, value: value or ctx.obj.app.conf.worker_prefetch_multiplier,
+              callback=lambda ctx, _,
+              value: value or ctx.obj.app.conf.worker_prefetch_multiplier,
               cls=CeleryOption,
               help_group="Worker Options",
-              help="Set custom prefetch multiplier value"
+              help="Set custom prefetch multiplier value "
                    "for this worker instance.")
 @click.option('-c',
               '--concurrency',
               type=int,
               metavar="<concurrency>",
-              callback=lambda ctx, _, value: value or ctx.obj.app.conf.worker_concurrency,
+              callback=lambda ctx, _,
+              value: value or ctx.obj.app.conf.worker_concurrency,
               cls=CeleryOption,
               help_group="Pool Options",
               help="Number of child processes processing the queue.  "
                    "The default is the number of CPUs available"
-                   "on your system.")
+                   " on your system.")
 @click.option('-P',
               '--pool',
               default='prefork',
@@ -178,6 +204,7 @@ def detach(path, argv, logfile=None, pidfile=None, uid=None,
               '--task-events',
               '--events',
               is_flag=True,
+              default=None,
               cls=CeleryOption,
               help_group="Pool Options",
               help="Send task-related events that can be captured by monitors"
@@ -231,15 +258,15 @@ def detach(path, argv, logfile=None, pidfile=None, uid=None,
               cls=CeleryOption,
               help_group="Queue Options")
 @click.option('--without-gossip',
-              default=False,
+              is_flag=True,
               cls=CeleryOption,
               help_group="Features")
 @click.option('--without-mingle',
-              default=False,
+              is_flag=True,
               cls=CeleryOption,
               help_group="Features")
 @click.option('--without-heartbeat',
-              default=False,
+              is_flag=True,
               cls=CeleryOption,
               help_group="Features", )
 @click.option('--heartbeat-interval',
@@ -259,20 +286,25 @@ def detach(path, argv, logfile=None, pidfile=None, uid=None,
 @click.option('-s',
               '--schedule-filename',
               '--schedule',
-              callback=lambda ctx, _, value: value or ctx.obj.app.conf.beat_schedule_filename,
+              callback=lambda ctx, _,
+              value: value or ctx.obj.app.conf.beat_schedule_filename,
               cls=CeleryOption,
               help_group="Embedded Beat Options")
 @click.option('--scheduler',
               cls=CeleryOption,
               help_group="Embedded Beat Options")
 @click.pass_context
+@handle_preload_options
 def worker(ctx, hostname=None, pool_cls=None, app=None, uid=None, gid=None,
            loglevel=None, logfile=None, pidfile=None, statedb=None,
            **kwargs):
     """Start worker instance.
 
+    \b
     Examples
     --------
+
+    \b
     $ celery --app=proj worker -l INFO
     $ celery -A proj worker -l INFO -Q hipri,lopri
     $ celery -A proj worker --concurrency=4
@@ -280,52 +312,49 @@ def worker(ctx, hostname=None, pool_cls=None, app=None, uid=None, gid=None,
     $ celery worker --autoscale=10,0
 
     """
-    app = ctx.obj.app
-    if ctx.args:
-        try:
-            app.config_from_cmdline(ctx.args, namespace='worker')
-        except (KeyError, ValueError) as e:
-            # TODO: Improve the error messages
-            raise click.UsageError(
-                "Unable to parse extra configuration from command line.\n"
-                f"Reason: {e}", ctx=ctx)
-    if kwargs.get('detach', False):
-        params = ctx.params.copy()
-        params.pop('detach')
-        params.pop('logfile')
-        params.pop('pidfile')
-        params.pop('uid')
-        params.pop('gid')
-        umask = params.pop('umask')
-        workdir = ctx.obj.workdir
-        params.pop('hostname')
-        executable = params.pop('executable')
-        argv = ['-m', 'celery', 'worker']
-        for arg, value in params.items():
-            if isinstance(value, bool) and value:
-                argv.append(f'--{arg}')
-            else:
-                if value is not None:
-                    argv.append(f'--{arg}')
-                    argv.append(str(value))
+    try:
+        app = ctx.obj.app
+        if ctx.args:
+            try:
+                app.config_from_cmdline(ctx.args, namespace='worker')
+            except (KeyError, ValueError) as e:
+                # TODO: Improve the error messages
+                raise click.UsageError(
+                    "Unable to parse extra configuration from command line.\n"
+                    f"Reason: {e}", ctx=ctx)
+        if kwargs.get('detach', False):
+            argv = ['-m', 'celery'] + sys.argv[1:]
+            if '--detach' in argv:
+                argv.remove('--detach')
+            if '-D' in argv:
+                argv.remove('-D')
+            if "--uid" in argv:
+                argv.remove('--uid')
+            if "--gid" in argv:
+                argv.remove('--gid')
+
             return detach(sys.executable,
                           argv,
                           logfile=logfile,
                           pidfile=pidfile,
                           uid=uid, gid=gid,
-                          umask=umask,
-                          workdir=workdir,
+                          umask=kwargs.get('umask', None),
+                          workdir=kwargs.get('workdir', None),
                           app=app,
-                          executable=executable,
+                          executable=kwargs.get('executable', None),
                           hostname=hostname)
-        return
-    maybe_drop_privileges(uid=uid, gid=gid)
-    worker = app.Worker(
-        hostname=hostname, pool_cls=pool_cls, loglevel=loglevel,
-        logfile=logfile,  # node format handled by celery.app.log.setup
-        pidfile=node_format(pidfile, hostname),
-        statedb=node_format(statedb, hostname),
-        no_color=ctx.obj.no_color,
-        **kwargs)
-    worker.start()
-    return worker.exitcode
+
+        maybe_drop_privileges(uid=uid, gid=gid)
+        worker = app.Worker(
+            hostname=hostname, pool_cls=pool_cls, loglevel=loglevel,
+            logfile=logfile,  # node format handled by celery.app.log.setup
+            pidfile=node_format(pidfile, hostname),
+            statedb=node_format(statedb, hostname),
+            no_color=ctx.obj.no_color,
+            quiet=ctx.obj.quiet,
+            **kwargs)
+        worker.start()
+        ctx.exit(worker.exitcode)
+    except SecurityError as e:
+        ctx.obj.error(e.args[0])
+        ctx.exit(1)
