@@ -1,13 +1,12 @@
 """Custom maps, sets, sequences, and other data structures."""
-import sys
 import time
 from collections import OrderedDict as _OrderedDict
 from collections import deque
-from collections.abc import (Callable, Mapping, MutableMapping, MutableSet,
-                             Sequence)
+from collections.abc import Callable, Mapping, MutableMapping, MutableSet, Sequence
 from heapq import heapify, heappop, heappush
 from itertools import chain, count
 from queue import Empty
+from typing import Any, Dict, Iterable, List  # noqa
 
 from .functional import first, uniq
 from .text import match_case
@@ -21,9 +20,9 @@ except ImportError:
 try:
     from django.utils.functional import LazyObject, LazySettings
 except ImportError:
-    class LazyObject:  # noqa
+    class LazyObject:
         pass
-    LazySettings = LazyObject  # noqa
+    LazySettings = LazyObject
 
 __all__ = (
     'AttributeDictMixin', 'AttributeDict', 'BufferMap', 'ChainMap',
@@ -114,8 +113,7 @@ class AttributeDictMixin:
             raise AttributeError(
                 f'{type(self).__name__!r} object has no attribute {k!r}')
 
-    def __setattr__(self, key, value):
-        # type: (str, Any) -> None
+    def __setattr__(self, key: str, value) -> None:
         """`d[key] = value -> d.key = value`."""
         self[key] = value
 
@@ -193,26 +191,12 @@ class DictAttribute:
             yield getattr(self.obj, key)
     itervalues = _iterate_values
 
-    if sys.version_info[0] == 3:  # pragma: no cover
-        items = _iterate_items
-        keys = _iterate_keys
-        values = _iterate_values
-    else:
-
-        def keys(self):
-            # type: () -> List[Any]
-            return list(self)
-
-        def items(self):
-            # type: () -> List[Tuple[Any, Any]]
-            return list(self._iterate_items())
-
-        def values(self):
-            # type: () -> List[Any]
-            return list(self._iterate_values())
+    items = _iterate_items
+    keys = _iterate_keys
+    values = _iterate_values
 
 
-MutableMapping.register(DictAttribute)  # noqa: E305
+MutableMapping.register(DictAttribute)
 
 
 class ChainMap(MutableMapping):
@@ -340,7 +324,7 @@ class ChainMap(MutableMapping):
         # changes take precedence.
         # pylint: disable=bad-reversed-sequence
         #   Someone should teach pylint about properties.
-        return chain(*[op(d) for d in reversed(self.maps)])
+        return chain(*(op(d) for d in reversed(self.maps)))
 
     def _iterate_keys(self):
         # type: () -> Iterable
@@ -360,23 +344,9 @@ class ChainMap(MutableMapping):
     def bind_to(self, callback):
         self._observers.append(callback)
 
-    if sys.version_info[0] == 3:  # pragma: no cover
-        keys = _iterate_keys
-        items = _iterate_items
-        values = _iterate_values
-
-    else:  # noqa
-        def keys(self):
-            # type: () -> List[Any]
-            return list(self._iterate_keys())
-
-        def items(self):
-            # type: () -> List[Tuple[Any, Any]]
-            return list(self._iterate_items())
-
-        def values(self):
-            # type: () -> List[Any]
-            return list(self._iterate_values())
+    keys = _iterate_keys
+    items = _iterate_items
+    values = _iterate_values
 
 
 class ConfigurationView(ChainMap, AttributeDictMixin):
@@ -624,7 +594,7 @@ class LimitedSet:
                     break  # oldest item hasn't expired yet
                 self.pop()
 
-    def pop(self, default=None):
+    def pop(self, default=None) -> Any:
         # type: (Any) -> Any
         """Remove and return the oldest item, or :const:`None` when empty."""
         while self._heap:
@@ -657,10 +627,6 @@ class LimitedSet:
         # type: (Any) -> bool
         return self._data == other._data
 
-    def __ne__(self, other):
-        # type: (Any) -> bool
-        return not self.__eq__(other)
-
     def __repr__(self):
         # type: () -> str
         return REPR_LIMITED_SET.format(
@@ -696,7 +662,7 @@ class LimitedSet:
         return len(self._heap) * 100 / max(len(self._data), 1) - 100
 
 
-MutableSet.register(LimitedSet)  # noqa: E305
+MutableSet.register(LimitedSet)
 
 
 class Evictable:
@@ -704,20 +670,17 @@ class Evictable:
 
     Empty = Empty
 
-    def evict(self):
-        # type: () -> None
+    def evict(self) -> None:
         """Force evict until maxsize is enforced."""
         self._evict(range=count)
 
-    def _evict(self, limit=100, range=range):
-        # type: (int) -> None
+    def _evict(self, limit: int = 100, range=range) -> None:
         try:
             [self._evict1() for _ in range(limit)]
         except IndexError:
             pass
 
-    def _evict1(self):
-        # type: () -> None
+    def _evict1(self) -> None:
         if self._evictcount <= self.maxsize:
             raise IndexError()
         try:
@@ -779,8 +742,7 @@ class Messagebuffer(Evictable):
         # type: () -> int
         return self._len()
 
-    def __contains__(self, item):
-        # type: () -> bool
+    def __contains__(self, item) -> bool:
         return item in self.data
 
     def __reversed__(self):
@@ -797,7 +759,7 @@ class Messagebuffer(Evictable):
         return len(self)
 
 
-Sequence.register(Messagebuffer)  # noqa: E305
+Sequence.register(Messagebuffer)
 
 
 class BufferMap(OrderedDict, Evictable):
