@@ -1,16 +1,21 @@
 import errno
+import sys
 from datetime import datetime, timedelta
 from pickle import dumps, loads
 from unittest.mock import Mock, call, patch
 
 import pytest
-import pytz
 
 from celery import __version__, beat, uuid
 from celery.beat import BeatLazyFunc, event_t
 from celery.schedules import crontab, schedule
 from celery.utils.objects import Bunch
 
+if sys.version_info >= (3, 9):
+    from zoneinfo import ZoneInfo
+else:
+    from backports.zoneinfo import ZoneInfo
+
 
 class MockShelve(dict):
     closed = False
@@ -164,6 +169,7 @@ class mocked_schedule(schedule):
 
 always_due = mocked_schedule(True, 1)
 always_pending = mocked_schedule(False, 1)
+always_pending_left_10_milliseconds = mocked_schedule(False, 0.01)
 
 
 class test_Scheduler:
@@ -193,7 +199,9 @@ class test_Scheduler:
         foo.apply_async = Mock(name='foo.apply_async')
 
         scheduler = mScheduler(app=self.app)
-        scheduler.apply_async(scheduler.Entry(task=foo.name, app=self.app, args=None, kwargs=None))
+        scheduler.apply_async(
+            scheduler.Entry(
+                task=foo.name, app=self.app, args=None, kwargs=None))
         foo.apply_async.assert_called()
 
     def test_apply_async_with_null_args_set_to_none(self):
@@ -354,6 +362,12 @@ class test_Scheduler:
                       schedule=always_pending)
         assert scheduler.tick() == 1 - 0.010
 
+    def test_pending_left_10_milliseconds_tick(self):
+        scheduler = mScheduler(app=self.app)
+        scheduler.add(name='test_pending_left_10_milliseconds_tick',
+                      schedule=always_pending_left_10_milliseconds)
+        assert scheduler.tick() == 0.010 - 0.010
+
     def test_honors_max_interval(self):
         scheduler = mScheduler(app=self.app)
         maxi = scheduler.max_interval
@@ -425,9 +439,10 @@ class test_Scheduler:
         assert a.schedule['bar'].schedule._next_run_at == 40
 
     def test_when(self):
-        now_time_utc = datetime(2000, 10, 10, 10, 10, 10, 10, tzinfo=pytz.utc)
+        now_time_utc = datetime(2000, 10, 10, 10, 10,
+                                10, 10, tzinfo=ZoneInfo("UTC"))
         now_time_casey = now_time_utc.astimezone(
-            pytz.timezone('Antarctica/Casey')
+            ZoneInfo('Antarctica/Casey')
         )
         scheduler = mScheduler(app=self.app)
         result_utc = scheduler._when(
@@ -689,16 +704,19 @@ class test_PersistentScheduler:
                 'first_missed', 'first_missed',
                 last_run_at=now_func() - timedelta(minutes=2),
                 total_run_count=10,
+                app=self.app,
                 schedule=app_schedule['first_missed']['schedule']),
             'second_missed': beat.ScheduleEntry(
                 'second_missed', 'second_missed',
                 last_run_at=now_func() - timedelta(minutes=2),
                 total_run_count=10,
+                app=self.app,
                 schedule=app_schedule['second_missed']['schedule']),
             'non_missed': beat.ScheduleEntry(
                 'non_missed', 'non_missed',
                 last_run_at=now_func() - timedelta(minutes=2),
                 total_run_count=10,
+                app=self.app,
                 schedule=app_schedule['non_missed']['schedule']),
         }
 
@@ -721,7 +739,8 @@ class test_Service:
 
     def get_service(self):
         Scheduler, mock_shelve = create_persistent_scheduler()
-        return beat.Service(app=self.app, scheduler_cls=Scheduler), mock_shelve
+        return beat.Service(
+            app=self.app, scheduler_cls=Scheduler), mock_shelve
 
     def test_pickleable(self):
         s = beat.Service(app=self.app, scheduler_cls=Mock)
